-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp - all rights reserved.
-- 
-- For more scripts and sample code, check out 
-- 	http://www.SQLskills.com
--
-- Disclaimer - Thoroughly test this script, execute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

--drop database creditss1
--drop database creditss2

-- Before this script will work you will need to:
-- (1) Download the zip, unzip and restore the credit database. You
--		can find the zip on the Past Events page from SQLskills.com.
-- (2) Create the multiple filegroup and partitioned view scenario. 
--		This is script: "Credit with Partitioned Views.sql" 
-- Now, you can use this script!
-- NOTE: This script has many comments which are not commented out...
-- In other words, don't try to just "execute" this script, you should
-- work your way through it slowly!

USE Credit
go

sp_helpfile
go

CREATE DATABASE [CreditSS1] 
ON  PRIMARY 
( NAME = N'CreditData'
	, FILENAME = N'D:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS1Data.mdfss')
,
( NAME = N'ChargesStaging'
	, FILENAME = N'D:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS1ChargesStaging.ndfss')
,
( NAME = N'CreditPVFGFile1'
	, FILENAME = N'D:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS1PVFGFile1.ndfss')
,
( NAME = N'CreditPVFGFile2'
	, FILENAME = N'D:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS1PVFGFile2.ndfss')
,
( NAME = N'CreditPVFGFile3'
	, FILENAME = N'D:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS1PVFGFile3.ndfss')
,
( NAME = N'CreditPVFGFile4'
	, FILENAME = N'D:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS1PVFGFile4.ndfss')
AS SNAPSHOT OF Credit
GO

-- with no open trans, runs immediately
-- sizes of files (all SHOW the same - in sys.master_files, etc. but... on disk they're SPARSE)
--CreditData		128k
--CreditLog			n/a
--CreditPVFGFile1	64k
--CreditPVFGFile2	64k
--CreditPVFGFile3	64k
--CreditPVFGFile4	64k
--ChargesStaging	64k

-- change some of august (nicely isolated)
select * from chargepv where charge_dt >= '20040806' and charge_dt < '20040807'
--12992 rows
update ChargePV
	SET charge_amt = charge_amt + charge_no
	where charge_dt >= '20040806' and charge_dt < '20040807'
-- CreditPVFGFile3	768k (copy on write)

-- change some of august - but not commit...
select * from chargepv where charge_dt >= '20040807' and charge_dt < '20040808'
--12928 rows
BEGIN TRAN
update ChargePV
	SET charge_amt = charge_amt + charge_no
	where charge_dt >= '20040807' and charge_dt < '20040808'
-- CreditPVFGFile3	1.31mb
ROLLBACK TRAN
-- CreditPVFGFile3	1.31mb (copy before write, not reverted - yes, it's duplicated...but probably not a big deal)

select * 
from [CreditSS1].dbo.chargepv 
where charge_dt >= '20040807' and charge_dt < '20040808'

-- change some of august - but not commit and then go create another snapshot...
select * from chargepv where charge_dt >= '20040808' and charge_dt < '20040809'
-- 13760 rows
BEGIN TRAN
update ChargePV
	SET charge_amt = charge_amt + charge_no
	where charge_dt >= '20040808' and charge_dt < '20040809'
-- CreditPVFGFile3	1.93mb
-- Create another snapshot while the tran is in flight...
-- Will need to go to another window to do it!
--CREATE DATABASE [CreditSS2] 
--ON  PRIMARY 
--( NAME = N'CreditData'
--	, FILENAME = N'D:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS2Data.mdfss')
--,
--( NAME = N'ChargesStaging'
--	, FILENAME = N'D:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS2ChargesStaging.ndfss')
--,
--( NAME = N'CreditPVFGFile1'
--	, FILENAME = N'D:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS2PVFGFile1.ndfss')
--,
--( NAME = N'CreditPVFGFile2'
--	, FILENAME = N'D:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS2PVFGFile2.ndfss')
--,
--( NAME = N'CreditPVFGFile3'
--	, FILENAME = N'D:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS2PVFGFile3.ndfss')
--,
--( NAME = N'CreditPVFGFile4'
--	, FILENAME = N'D:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS2PVFGFile4.ndfss')
--AS SNAPSHOT OF Credit
--GO
--CreditData		512k
--CreditLog			n/a
--CreditPVFGFile1	64k
--CreditPVFGFile2	64k
--CreditPVFGFile3	768k
--CreditPVFGFile4	64k
--ChargesStaging	64k
ROLLBACK TRAN
--CreditData		512k
--CreditLog			n/a
--CreditPVFGFile1	64k
--CreditPVFGFile2	64k
--CreditPVFGFile3	768k
--CreditPVFGFile4	64k
--ChargesStaging	64k

-- Now that we have two snapshots - what happens with a modification?
-- change some of august
select * from chargepv where charge_dt >= '20040914' and charge_dt < '20040915'
--13344 rows
update ChargePV
	SET charge_amt = charge_amt + charge_no
	where charge_dt >= '20040914' and charge_dt < '20040915'
--SS1 - CreditPVFGFile4	704k
--SS2 - CredtiPVFGFile4	704k

select * from sys.master_files where database_id = db_id('credit')
select * from sys.master_files where database_id = db_id('creditSS1')
select * from sys.master_files where database_id = db_id('creditSS2')

-- revert back to unmodified version...
restore database credit from database_snapshot = 'creditss1'
--Msg 3137, Level 16, State 4, Line 1
--Database can not be reverted. Either the primary or the snapshot 
--names are improperly specified, all other snapshots have not 
--been dropped, or there are missing files.
--Msg 3013, Level 16, State 1, Line 1
--RESTORE DATABASE is terminating abnormally.
drop database creditss2
-- fast... :)
restore database credit from database_snapshot = 'creditss1'
-- works!
select * from credit.dbo.chargepv where charge_dt >= '20040806' and charge_dt < '20040807'
select * from creditss1.dbo.chargepv where charge_dt >= '20040806' and charge_dt < '20040807'
-- once you revert the image is restored AND the snapshot STAYS
SS1 - CreditPVFGFile3	1.93mb

-- change some of august AGAIN
select * from chargepv where charge_dt >= '20040806' and charge_dt < '20040807'
--12992 rows
update ChargePV
	SET charge_amt = 42
	where charge_dt >= '20040806' and charge_dt < '20040807'
-- not fast - 13sec!
SS1 - CreditPVFGFile3	1.93mb unchanged...

-- change some of august that was never changed...
select * from chargepv where charge_dt >= '20040816' and charge_dt < '20040817'
--13248 rows
update ChargePV
	SET charge_amt = 16
	where charge_dt >= '20040816' and charge_dt < '20040817'
-- better - 2sec!
SS1 - CreditPVFGFile3	2.56mb unchanged...

-- can we revert back...again?!
restore database credit from database_snapshot = 'creditss1'
-- works!
select * from credit.dbo.chargepv where charge_dt >= '20040816' and charge_dt < '20040817'
select * from creditss1.dbo.chargepv where charge_dt >= '20040816' and charge_dt < '20040817'
-- once you revert the image is restored AND the snapshot STAYS
SS1 - CreditPVFGFile3	2.56mb